# coding=utf_8

import pygame
import random
import math
from pygame.locals import *

def cachable(func):
	''' http://code.activestate.com/recipes/576365/ '''
	func.cache = {}
	def out(*args):
		if args not in func.cache:
			func.cache[args] = func(*args)
		return func.cache[args]
	return out


def resize(surface,size,smooth=True,inplace=False):
	size = (int(size[0]),int(size[1]))
	if smooth:
		if inplace:
			return pygame.transform.smoothscale(surface,size,surface)
		else:
			return pygame.transform.smoothscale(surface,size)
	else:
		return pygame.transform.scale(surface,size)

def flip(surface,vertical=False):
	if vertical:
		return pygame.transform.flip(surface,False,True)
	else:
		return pygame.transform.flip(surface,True,False)

def colorOverlay(surface,color=(128,128,128,128),blend=False):
	overlay_surface = _createOverlay(surface.get_size(),color)
	if blend:
		surface.blit(overlay_surface,(0,0),None,blend)
	else:
		overlay_surface = overlay_surface.convert_alpha()
		surface.blit(overlay_surface,(0,0))

@cachable
def _createOverlay(size,color):
	overlay_surface = pygame.Surface(size)
	overlay_surface.fill(color)
	return overlay_surface

def createGradient(size,from_color,to_color,vertical=False,ratio=1.0):
	srf = pygame.Surface(size).convert_alpha()
	srf.fill((0,0,0,0))
	srf.lock()
	if vertical:
		st = size[1]
	else:
		st = size[0]
	dr = to_color[0]-from_color[0]
	dg = to_color[1]-from_color[1]
	db = to_color[2]-from_color[2]
	da = to_color[3]-from_color[3]
		
	for n in xrange(st):
		if vertical:
			x1 = 0
			x2 = size[0]
			y1 = n
			y2 = n
		else:
			x1 = n
			x2 = n
			y1 = 0
			y2 = size[1]
		delta = (n/float(st))
		nn = delta**ratio
		cr = from_color[0]+(dr*nn)
		cg = from_color[1]+(dg*nn)
		cb = from_color[2]+(db*nn)
		ca = from_color[3]+(da*nn)
		if cr>255:cr=255
		if cg>255:cg=255
		if cb>255:cb=255
		if ca>255:ca=255
		if cr<0:cr=0
		if cg<0:cg=0
		if cb<0:cb=0
		if ca<0:ca=0
		color = (int(cr),int(cg),int(cb),int(ca))
		pygame.draw.line(srf,color,(x1,y1),(x2,y2))
	srf.unlock()
	return srf
	
def scratch(srf,row_size=3,offset=10,phase=1.0):
	siz = srf.get_size()
	sizlines = siz[1]
	w = siz[0]
	h = row_size
	for line in xrange(sizlines/row_size):
		y = line*row_size
		oy = y
		x = round(((offset*2) * random.random())-offset)*phase
		ox = 0
		srf.blit(srf,(int(x),y),(ox,oy,w,h))
	
def wave(surface,amp,wphase):
	siz = surface.get_size()
	for line in xrange(siz[1]):
		num = math.sin( (wphase+line)*0.1 )
		y = line
		x = amp*num
		surface.blit(surface,(int(x),y),(0,y,siz[0],1))

def rotate(surface,angle):
	return pygame.transform.rotate(surface, angle)
	
def shake(surface,offset,phase):
	v = offset*phase
	rx = random.random()
	ry = random.random()
	if rx > 0.5:rx = 1
	if ry > 0.5:ry = 1
	if rx < 0.5:rx = -1
	if ry < 0.5:ry = -1
	x = int( (rx*v)/2 )
	y = int( (ry*v) )
	surface.blit(surface,(x,y))
	return surface
	
def change_alpha(surface,alpha=0.5):
	surface = surface.convert()
	surface.set_alpha(int(255*alpha))
	return surface

def colorize(dat,cl,rgba=False):
	''' neprilis rychly kolorovac (jen pro male srf) - pygame '''
	siz = dat.get_size()
	srf = pygame.Surface(siz).convert_alpha()
	if rgba:
		srf.fill((0,0,0,0))
	else:
		srf.fill((0,0,0))
	for y in xrange(siz[1]):
		for x in xrange(siz[0]):
			clr = dat.get_at((x,y))
			nw = ((clr[0]+clr[1]+clr[2])/3)
			r,g,b = (int(nw*cl[0]),int(nw*cl[1]),int(nw*cl[2]))
			if rgba:
				srf.set_at((x,y),(r,g,b,clr[3]))
			else:
				srf.set_at((x,y),(r,g,b))
	return srf	

def blur(surface,value=1,norand=False):
	''' Pseudo BLUR func. '''
	if value < 0.1:value = 0.1
	if value == 1.0:value = 1.05
	rnd = (random.random()*0.1)
	if norand:rnd = 0.1
	value = 1.0/value+rnd
	original_size = surface.get_size()
	new_size = [ int(original_size[0]*value), int(original_size[1]*value) ]
	if new_size[0] < 3:new_size[0] = 3
	if new_size[1] < 3:new_size[1] = 3
	surface_small = pygame.transform.smoothscale(surface,new_size)
	pygame.transform.smoothscale(surface_small,original_size,surface)

def blur2(surface,cycles=2,bvalue=2,norand=False):
	for n in xrange(cycles):
		blur(surface,bvalue,norand)

def bloom(surface,value=4,norand=False):
	'''  '''
	srf = surface.copy()
	blur(srf,value,norand)
	surface.blit(srf,(0,0),None,BLEND_RGB_ADD)

def bloomMult(surface,value=4,norand=False):
	'''  '''
	srf = surface.copy()
	blur(srf,value,norand)
	surface.blit(srf,(0,0),None,BLEND_RGB_MULT)

def deform(surface,phase=1.0,size=50,col=3,row=3,blur_value=2,norand=False):
	sw,sh = surface.get_size()
	p = phase
	if col:
		cols = int(math.ceil((sw/2)/float(col)))
		for c in xrange(cols):
			o = math.sin( ( (c/float(cols)) )*math.pi )*p
			ox = (c*col)
			x = ox-(o*size)
			surface.blit(surface,(x,0),(ox,0,col,sh))
			ox = (sw)-(c*col)
			x = ox+(o*size)
			surface.blit(surface,(x,0),(ox,0,col,sh))
	if row:
		rows = int(math.ceil((sh/2)/float(row)))
		for r in xrange(rows):
			o = math.sin( ( (r/float(rows)) )*math.pi )*p
			oy = (r*row)
			y = oy-(o*size)
			surface.blit(surface,(0,y),(0,oy,sw,row))
			oy = (sh)-(r*row)
			y = oy+(o*size)
			surface.blit(surface,(0,y),(0,oy,sw,row))
	blur2(surface,int(1+(blur_value*p)),1+(3*p),norand)

def deform2(surface,phase=1.0,size=50,recs=40):
	sw,sh = surface.get_size()
	p = math.sin(phase*math.pi*2)
	srf = surface.copy()
	dx = sw
	dy = sh
	r1 = random.random()
	r2 = random.random()
	for n in xrange(recs):
		q = math.sin((phase+(n/float(recs)))*math.pi)
		w = dx
		h = dy
		x = ((sw-dx)/2)+(size*(p*q*r1))
		y = ((sh-dy)/3)+(size*(p*q*r2))
		#y = ((sh-dy)/2)
		ox = (sw-dx)/2
		oy = (sh-dy)/3
		surface.blit(srf,(x,y),(ox,oy,w,h))
		dx = dx*0.97
		dy = dy*0.97
	
